@@ -80,11 +80,42 @@ DEFINE_SEMANTIC_ITEM(false, false, KaxTrackTranslate)
 DEFINE_SEMANTIC_ITEM(false, true, KaxTrackAudio)
 DEFINE_SEMANTIC_ITEM(false, true, KaxTrackVideo)
 DEFINE_SEMANTIC_ITEM(false, true, KaxContentEncodings)
+#if MATROSKA_VERSION >= 2
+DEFINE_SEMANTIC_ITEM(false, true, KaxTrackOperation)
+#endif
 DEFINE_END_SEMANTIC(KaxTrackEntry)
 
 DEFINE_MKX_MASTER     (KaxTracks, 0x1654AE6B, 4, KaxSegment, "Tracks");
 DEFINE_MKX_MASTER_CONS(KaxTrackEntry,   0xAE, 1, KaxTracks, "TrackEntry");
 
+#if MATROSKA_VERSION >= 2
+DEFINE_START_SEMANTIC(KaxTrackOperation)
+DEFINE_SEMANTIC_ITEM(false, true, KaxTrackCombinePlanes)
+DEFINE_SEMANTIC_ITEM(false, true, KaxTrackJoinBlocks)
+DEFINE_END_SEMANTIC(KaxTrackOperation)
+
+DEFINE_START_SEMANTIC(KaxTrackCombinePlanes)
+DEFINE_SEMANTIC_ITEM(true, false, KaxTrackPlane)
+DEFINE_END_SEMANTIC(KaxTrackCombinePlanes)
+
+DEFINE_START_SEMANTIC(KaxTrackPlane)
+DEFINE_SEMANTIC_ITEM(true, true, KaxTrackPlaneUID)
+DEFINE_SEMANTIC_ITEM(true, true, KaxTrackPlaneType)
+DEFINE_END_SEMANTIC(KaxTrackPlane)
+
+DEFINE_START_SEMANTIC(KaxTrackJoinBlocks)
+DEFINE_SEMANTIC_ITEM(true, false, KaxTrackJoinUID)
+DEFINE_END_SEMANTIC(KaxTrackJoinBlocks)
+
+DEFINE_MKX_MASTER  (KaxTrackOperation,     0xE2, 1, KaxTracks, "TrackOperation");
+DEFINE_MKX_MASTER  (KaxTrackCombinePlanes, 0xE3, 1, KaxTrackOperation, "TrackCombinePlanes");
+DEFINE_MKX_MASTER  (KaxTrackPlane,         0xE4, 1, KaxTrackCombinePlanes, "TrackPlane");
+DEFINE_MKX_UINTEGER(KaxTrackPlaneUID,      0xE5, 1, KaxTrackPlane, "TrackPlaneUID");
+DEFINE_MKX_UINTEGER(KaxTrackPlaneType,     0xE6, 1, KaxTrackPlane, "TrackPlaneType");
+DEFINE_MKX_MASTER  (KaxTrackJoinBlocks,    0xE9, 1, KaxTrackOperation, "TrackJoinBlocks");
+DEFINE_MKX_UINTEGER(KaxTrackJoinUID,       0xED, 1, KaxTrackJoinBlocks, "TrackJoinUID");
+#endif
+
 KaxTrackEntry::KaxTrackEntry(EBML_EXTRA_DEF)
 	:EbmlMaster(EBML_CLASS_SEMCONTEXT(KaxTrackEntry) EBML_DEF_SEP EBML_EXTRA_CALL)
 	,bGlobalTimecodeScaleIsSet(false)
